<?php // $Id$

class geocode {
  var $result;
  var $formatted_address;

  var $point;
  var $box;
  var $street1;
  var $street2;
  var $street_number;
  var $city;
  var $state;
  var $zip;
  var $country;
  var $country_name;


  function set_result($value) {
    $this->$result = $value;
  }

  function get_result($type, $item = NULL) {
    $func = 'get_'. $type;
    if (method_exists($this, $func)) {
      return $this->$func($item);
    }
  }

  function get_point() {
    return $this->point;
  }

  function get_linestring() {
    return $this->linestring;
  }

  function get_wkt($item) {
    // TODO figure out how to make any geo data type (shape, line) into a point.
    switch ($item) {
      case 'point':
        return 'POINT('. $this->point['lon'].' '. $this->point['lat'] .')';

      case 'linestring':
        $wkt = 'LINESTRING(';
        foreach ($this->linestring as $point) {
          $wkt .= $point['lon'] .' '. $point['lat'] .', ';
        }
        $wkt = substr($wkt, 0, -2) .')';
        return $wkt;
    }
  }

  function get_field_text($item) {
    return array('value' => $this->$item);
  }

  function get_field_postal() {
    $postal = array();
    foreach (array('street_number', 'street1', 'street2', 'city', 'state', 'zip', 'country') as $item) {
      if (isset($this->$item))
        $postal[$item] = $this->$item;
    }
    return $postal;
  }

  /**
   * Gets the human-readable address of the requested location.
   *
   * @return
   *   A string containing the human-readable address of this location.
   */
  function get_formatted_address() {
    return $this->formatted_address;
  }
}

class geocode_google extends geocode {


  function set_result($result) {
    $this->result = $result;

    // Store the formatted address if present.
    $this->formatted_address = $result->formatted_address;

    // Translate the values returned by the Google Geocoding API to values used
    // by the Geocode module.
    foreach ($result->address_components as $component) {
      $type = $component->types[0];
      switch ($type) {
        case 'postal_code':
          $this->zip = $component->long_name;
          break;
        case 'country':
          $this->country = $component->short_name;
          $this->country_name = $component->long_name;
          break;
        case 'administrative_area_level_1':
          $this->state = $component->long_name;
          break;
        case 'locality':
          $this->city = $component->long_name;
          break;
        case 'route':
          $this->street1 = $component->long_name;
          break;
        case 'street_number':
          $this->street_number = $component->long_name;
          break;
        default:
          break;
      }
    }

    $this->point = array(
     'type' => 'point',
     'lat' => $result->geometry->location->lat,
     'lon' => $result->geometry->location->lng,
    );

    // Convert the new viewport data into the old north-east-south-west format
    // to maintain backward compatibility.
    if (isset($result->geometry->viewport)) {
      $this->box = array(
        'north' => $result->geometry->viewport->northeast->lat,
        'east' => $result->geometry->viewport->northeast->lng,
        'south' => $result->geometry->viewport->southwest->lat,
        'west' => $result->geometry->viewport->southwest->lng,
      );
    }
  }

  /**
   * Executes a geocoding request against Google's geocoding webservice.
   * 
   * @param $input
   *   The location or address that needs to be geocoded. Can be either a string
   *   or an array containing multiple locations.
   * @param $options
   *   An array of optional options to pass to the geocoding webservice.
   * 
   * @return
   *   TRUE if the geocoding request was successful, FALSE otherwise.
   */
  function geocode($input, $options) {
    if (is_array($input)) {
      $input = join(',', array_filter($input));
      $input = str_replace(' ', '+', $input);
    }

    $query = array(
      'address' => $input,
      'sensor' => 'false',
    );

    $url = url('http://maps.google.com/maps/api/geocode/json', array('query' => $query));
    $geocode_result = drupal_http_request($url);
    $result = json_decode($geocode_result->data);

    if ($result->status == 'OK') {
      $this->set_result($result->results[0]);
      return TRUE;
    }

    return FALSE;
  }
}
